/* 
 * Copyright (C) 1999 Canek Pelez Valds <canek@abulafia.fciencias.unam.mx>
 * Copyright (C) 2007 Manuel Teira Paz <manuel.teira@telefonica.net>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "gpe-snes.h"

#include <gpe/pixmaps.h>
#include <gpe/init.h>

static void main_window(GlobalPrefs *prefs);
static GlobalPrefs* app_init();
static void get_roms_window(GsnesData *gsnes_data);
static void get_image_window(GsnesData *gsnes_data);
static GtkWidget* get_list_roms(GsnesData *data);
static void create_menus(GsnesData *gsnes_data, GtkWidget *container);
static void create_toolbar(GsnesData *gsnes_data, GtkWidget *container);

struct gpe_icon my_icons[] = {
    {"gsnes", PIXMAPDIR "/gpe-snes.png"},
    {NULL, NULL}
};

int main(int argc, char *argv[])
{
    if (gpe_application_init(&argc, &argv) == FALSE) {
        return 1;
    }

    if (gpe_load_icons(my_icons) == FALSE) {
        return 1;
    }
	
#ifdef ENABLE_NLS
    setlocale(LC_CTYPE, "");
    bindtextdomain(PACKAGE, LOCALEDIR);
    textdomain(PACKAGE);
#endif

    main_window(app_init());
    gtk_main();
    return 0;
}

void main_window(GlobalPrefs *prefs)
{
    gchar *status;
    GtkWidget *vbox;
    GtkWidget *hsep;
    GsnesData *data = gsnes9x_data_new();
    
    data->prefs = prefs;
    data->roms_list = g_list_first(get_roms_list());
    data->app = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    data->status = gtk_label_new("");
	
    gtk_misc_set_alignment(GTK_MISC(data->status), 0, 0.5);
	
    g_object_set_data(G_OBJECT(data->app),
                      "gsnes_data",
                      (gpointer) data);

    gpe_set_window_icon(data->app, "gsnes");

    gtk_widget_realize(data->app);
    g_signal_connect(G_OBJECT(data->app), "delete_event",
                     G_CALLBACK(quit_by_close_cb), NULL);

    data->paned = gtk_hpaned_new ();
	
    get_roms_window(data);
    get_image_window(data);

    gtk_paned_add1(GTK_PANED(data->paned), data->roms_w);
    gtk_paned_add2(GTK_PANED(data->paned), data->image_w);

    vbox = gtk_vbox_new(FALSE, 0);
    hsep = gtk_hseparator_new();

    create_menus(data, vbox);
    create_toolbar(data, vbox);

    gtk_box_pack_start(GTK_BOX(vbox), data->paned, TRUE, TRUE,   0);
    gtk_box_pack_start(GTK_BOX(vbox), hsep, FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox), data->status, FALSE, FALSE, 0);

    GList *list;
    for (list = g_list_first(data->rom_widgets);
         list != NULL; list = g_list_next(list)) {
        gtk_widget_set_sensitive(GTK_WIDGET(list->data), FALSE);
    }

    gtk_container_add(GTK_CONTAINER(data->app), vbox);

    gtk_widget_show(data->image_w);
    gtk_widget_show(data->roms_w);
    gtk_widget_show(data->paned);
    gtk_widget_show(hsep);
    gtk_widget_show(data->status);
    gtk_widget_show(data->app);
    gtk_widget_show(vbox);

    gtk_widget_grab_focus(data->clist);
	
    status = g_strdup_printf(_("Total ROMs: %d"),
                             g_list_length(g_list_first(data->roms_list)));
    gtk_label_set_text(GTK_LABEL(data->status), status);
    g_free(status);

    g_signal_connect(G_OBJECT(data->app), "check-resize",
                     G_CALLBACK(resize_pixmap_cb),
                     (gpointer)data);
}

GlobalPrefs* app_init()
{
    GlobalPrefs *prefs = g_new(GlobalPrefs, 1);

    F_NAME();
	
    prefs = read_conf_file();
	
    if (!prefs->string_prefs[SNES9X_COMMAND-ROM_STR_PREFS]) {
        g_warning(_("The global configuration file is"
                    "corrupted or incomplete. Recreating "
                    "it with default values."));
        g_free(prefs);
        create_conf_file();
        return app_init();
    }

    return prefs;
}

void get_roms_window(GsnesData *gsnes_data)
{
    gsnes_data->roms_w = gtk_scrolled_window_new(NULL, NULL);

    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(gsnes_data->roms_w),
                                   GTK_POLICY_NEVER,
                                   GTK_POLICY_AUTOMATIC);
	
    gsnes_data->clist = get_list_roms(gsnes_data);
	
    g_signal_connect(G_OBJECT(gsnes_data->clist), 
                     "select_row",
                     G_CALLBACK(select_rom_cb),
                     (gpointer) gsnes_data);

    g_signal_connect(G_OBJECT(gsnes_data->clist),
                     "button-release-event",
                     G_CALLBACK(right_select_rom_cb),
                     (gpointer) gsnes_data);
	
    g_signal_connect(G_OBJECT(gsnes_data->clist), 
                     "key-press-event",
                     G_CALLBACK(press_key_cb),
                     (gpointer) gsnes_data);

    gtk_container_add(GTK_CONTAINER(gsnes_data->roms_w),
                      gsnes_data->clist);

    gtk_widget_show(gsnes_data->clist);

    gtk_widget_set_size_request(gsnes_data->roms_w, 
                                IMAGE_WIDTH / 2, IMAGE_HEIGHT / 2);
}

void get_image_window(GsnesData *gsnes_data)
{
    gsnes_data->image_w = gtk_scrolled_window_new(NULL, NULL);
	
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(gsnes_data->image_w),
                                   GTK_POLICY_NEVER,
                                   GTK_POLICY_NEVER);

    gsnes_data->pixmap = get_image_pixmap(DEFAULT_IMAGE, 
                                          IMAGE_WIDTH, IMAGE_HEIGHT);

    g_signal_connect(G_OBJECT(gsnes_data->image_w),
                     "button-press-event",
                     G_CALLBACK(right_click_pixmap_cb),
                     (gpointer) gsnes_data);

    gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(gsnes_data->image_w),
                                          gsnes_data->pixmap);
        
    gtk_widget_show(gsnes_data->pixmap);

    gtk_widget_set_size_request(gsnes_data->image_w, 
                                IMAGE_WIDTH+20, IMAGE_HEIGHT+20);
}

GtkWidget* get_list_roms(GsnesData *data)
{
    guint i, num;
    GList *list = data->roms_list;
    GtkWidget *clist;
    gchar **row;

    F_NAME();

    row = g_new(gchar*, 1);
	
    list = g_list_first(list);
    num = g_list_length(list);
    clist = gtk_clist_new(1);

    gtk_clist_set_selection_mode(GTK_CLIST(clist),
                                 GTK_SELECTION_SINGLE);
    gtk_clist_set_auto_sort(GTK_CLIST(clist), TRUE);
	
    for (i = 0; i < num; i++) {
        row[0] = g_strdup(((ROMInfo*)(list->data))->string_prefs[NAME]);
        gtk_clist_append(GTK_CLIST(clist), row);
        list = g_list_next(list);
        g_free(row[0]);
    }
	
    g_free(row);
    return clist;
}


static void create_menus(GsnesData *gsnes_data, GtkWidget *container)
{
    GtkWidget *popmenu = gtk_menu_new();
    GtkWidget *mainmenu = gtk_menu_bar_new();
    GtkWidget *play_item = gtk_menu_item_new_with_label(_("Play ROM"));
    GtkWidget *remove_item = gtk_menu_item_new_with_label(_("Remove ROM"));
    GtkWidget *prefs_item = gtk_menu_item_new_with_label(_("ROM Preferences"));
    GtkWidget *menu;
    GtkWidget *mitem;


    gtk_box_pack_start(GTK_BOX(container), mainmenu, FALSE, FALSE, 0);
    gtk_widget_show(mainmenu);

    menu = gtk_menu_new();
    mitem = gtk_menu_item_new_with_mnemonic(_("_File"));
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(mitem), menu);
    gtk_menu_shell_append(GTK_MENU_SHELL(mainmenu), mitem);
    gtk_widget_show(mitem);

    mitem = gtk_image_menu_item_new_with_mnemonic(_("_Add new ROM"));
    g_signal_connect(G_OBJECT(mitem), "activate",
                     G_CALLBACK(add_cb), 
                     (gpointer) gsnes_data);
    gtk_widget_show(mitem);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mitem);

    mitem = gtk_image_menu_item_new_with_mnemonic(_("Re_move ROM"));
    g_signal_connect(G_OBJECT(mitem), "activate",
                     G_CALLBACK(remove_cb), 
                     (gpointer) gsnes_data);
    gtk_widget_show(mitem);
    gsnes_data->rom_widgets = 
        g_list_append(gsnes_data->rom_widgets, (gpointer) mitem);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mitem);


    mitem = gtk_image_menu_item_new_with_mnemonic(_("_Play ROM"));
    g_signal_connect(G_OBJECT(mitem), "activate",
                     G_CALLBACK(play_cb), 
                     (gpointer) gsnes_data);
    gtk_widget_show(mitem);
    gsnes_data->rom_widgets = 
        g_list_append(gsnes_data->rom_widgets, (gpointer) mitem);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mitem);

    mitem = gtk_image_menu_item_new_with_mnemonic(_("_Quit"));
    g_signal_connect(G_OBJECT(mitem), "activate",
                     G_CALLBACK(quit_by_menu_cb), 
                     (gpointer) gsnes_data);
    gtk_widget_show(mitem);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mitem);


    menu = gtk_menu_new();
    mitem = gtk_menu_item_new_with_mnemonic(_("_Edit"));
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(mitem), menu);
    gtk_menu_shell_append(GTK_MENU_SHELL(mainmenu), mitem);
    gtk_widget_show(mitem);


    mitem = gtk_image_menu_item_new_with_mnemonic(_("_ROM Preferences"));
    g_signal_connect(G_OBJECT(mitem), "activate",
                     G_CALLBACK(rom_preferences_cb), 
                     (gpointer) gsnes_data);
    gtk_widget_show(mitem);
    gsnes_data->rom_widgets = 
        g_list_append(gsnes_data->rom_widgets, (gpointer) mitem);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mitem);

    mitem = gtk_image_menu_item_new_with_mnemonic(_("_Global Preferences"));
    g_signal_connect(G_OBJECT(mitem), "activate",
                     G_CALLBACK(global_preferences_cb), 
                     (gpointer) gsnes_data);
    gtk_widget_show(mitem);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mitem);
        
    menu = gtk_menu_new();
    mitem = gtk_menu_item_new_with_mnemonic(_("_Help"));
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(mitem), menu);
    gtk_menu_shell_append(GTK_MENU_SHELL(mainmenu), mitem);
    gtk_widget_show(mitem);
        
    mitem = gtk_image_menu_item_new_with_mnemonic(_("_About"));
    g_signal_connect(G_OBJECT(mitem), "activate",
                     G_CALLBACK(about_cb),
                     (gpointer) gsnes_data);
    gtk_widget_show(mitem);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mitem);

    gtk_menu_shell_append(GTK_MENU_SHELL(popmenu), play_item);
    gtk_menu_shell_append(GTK_MENU_SHELL(popmenu), remove_item);
    gtk_menu_shell_append(GTK_MENU_SHELL(popmenu), prefs_item);
    gtk_widget_show(play_item);
    gtk_widget_show(remove_item);
    gtk_widget_show(prefs_item);

    gtk_menu_attach_to_widget(GTK_MENU(popmenu),
                              gsnes_data->clist,
                              popup_menu_detacher);
        
    g_signal_connect(G_OBJECT(play_item), "activate",
                     G_CALLBACK(play_cb),
                     (gpointer) gsnes_data);
    g_signal_connect(G_OBJECT(remove_item), "activate",
                     G_CALLBACK(remove_cb),
                     (gpointer) gsnes_data);
    g_signal_connect(G_OBJECT(prefs_item), "activate",
                     G_CALLBACK(rom_preferences_cb),
                     (gpointer) gsnes_data);
    gsnes_data->popmenu = popmenu;
    gtk_widget_show(popmenu);
}

static void create_toolbar(GsnesData *gsnes_data, GtkWidget *container)
{
    GtkWidget *toolbar;
    GtkWidget *pw;

    toolbar = gtk_toolbar_new();
    gtk_toolbar_set_orientation(GTK_TOOLBAR(toolbar), 
                                GTK_ORIENTATION_HORIZONTAL);
    gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
    GTK_WIDGET_UNSET_FLAGS(toolbar, GTK_CAN_FOCUS);
    gtk_box_pack_start(GTK_BOX(container), toolbar, FALSE, FALSE, 0);
    gtk_widget_show(toolbar);

    GtkToolItem *item;

    pw = gtk_image_new_from_stock(GTK_STOCK_NEW,
                                  gtk_toolbar_get_icon_size(GTK_TOOLBAR(toolbar)));
    gtk_widget_show(GTK_WIDGET(pw));
    item = gtk_tool_button_new(pw, _("New ROM"));
    g_signal_connect(G_OBJECT(item), "clicked", 
                     G_CALLBACK(add_cb), 
                     (gpointer) gsnes_data);
    gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
    GTK_WIDGET_UNSET_FLAGS(item, GTK_CAN_FOCUS);
    gtk_widget_show(GTK_WIDGET(item));

    pw = gtk_image_new_from_stock(GTK_STOCK_DELETE,
                                  gtk_toolbar_get_icon_size(GTK_TOOLBAR(toolbar)));
    gtk_widget_show(GTK_WIDGET(pw));
    item = gtk_tool_button_new(pw, _("Remove ROM"));
    g_signal_connect(G_OBJECT(item), "clicked",
                     G_CALLBACK(remove_cb),
                     (gpointer) gsnes_data);
    gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
    gsnes_data->rom_widgets = 
        g_list_append(gsnes_data->rom_widgets, (gpointer) item);
    GTK_WIDGET_UNSET_FLAGS(item, GTK_CAN_FOCUS);
    gtk_widget_show(GTK_WIDGET(item));

    pw = gtk_image_new_from_stock(GTK_STOCK_MEDIA_PLAY,
                                  gtk_toolbar_get_icon_size(GTK_TOOLBAR(toolbar)));
    gtk_widget_show(GTK_WIDGET(pw));
    item = gtk_tool_button_new(pw, _("Play ROM"));
    g_signal_connect(G_OBJECT(item), "clicked",
                     G_CALLBACK(play_cb),
                     (gpointer) gsnes_data);
    gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
    gsnes_data->rom_widgets = 
        g_list_append(gsnes_data->rom_widgets, (gpointer) item);
    GTK_WIDGET_UNSET_FLAGS(item, GTK_CAN_FOCUS);
    gtk_widget_show(GTK_WIDGET(item));

    pw = gtk_image_new_from_stock(GTK_STOCK_PREFERENCES,
                                  gtk_toolbar_get_icon_size(GTK_TOOLBAR(toolbar)));
    gtk_widget_show(GTK_WIDGET(pw));
    item = gtk_tool_button_new(pw, _("Preferences"));
    g_signal_connect(G_OBJECT(item), "clicked",
                     G_CALLBACK(global_preferences_cb),
                     (gpointer) gsnes_data);
    gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
    GTK_WIDGET_UNSET_FLAGS(item, GTK_CAN_FOCUS);
    gtk_widget_show(GTK_WIDGET(item));

    pw = gtk_image_new_from_stock(GTK_STOCK_QUIT,
                                  gtk_toolbar_get_icon_size(GTK_TOOLBAR(toolbar)));
    gtk_widget_show(GTK_WIDGET(pw));
    item = gtk_tool_button_new(pw, _("Quit"));
    g_signal_connect(G_OBJECT(item), "clicked",
                     G_CALLBACK(quit_by_menu_cb),
                     (gpointer) gsnes_data);
    gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
    GTK_WIDGET_UNSET_FLAGS(item, GTK_CAN_FOCUS);
    gtk_widget_show(GTK_WIDGET(item));


}    

/* main.c ends here. */
