#include "randr.h"

#ifdef HAVE_RANDR
#include <gdk/gdkdisplay.h>
#include <gdk/gdkscreen.h>
#include <gdk/gdkx.h>
#include <X11/extensions/Xrandr.h>
#endif

static int saved_w = -1;
static int saved_h = -1;

#ifdef HAVE_RANDR
static int bestsize(XRRScreenSize *sizes, int nsizes, int width, int height)
{
    int best, best_w, best_h;
    int i;

    best = 0;
    best_w = sizes[0].width;
    best_h = sizes[0].height;
    for (i = 1; i < nsizes; i++) {
        if (ABS(sizes[i].width - width) < ABS(best_w - width) ||
            (sizes[i].width == best_w &&
             ABS(sizes[i].height - height) < ABS(best_h - height))) {
            best = i;
            best_w = sizes[i].width;
            best_h = sizes[i].height;
	}
    }
    g_message("Best mode for %dx%d is %dx%d\n", width, height, 
              sizes[best].width, sizes[best].height);
    return best;
}
#endif


void set_rr_mode(int width, int height)
{
#ifdef HAVE_RANDR
    GdkDisplay *display;
    Display *xdisplay;
    int major, minor;
    int event_base, error_base;
    GdkScreen *screen;
    GdkWindow *root_window;
    XRRScreenConfiguration *config;
    XRRScreenSize *sizes;
    int n_sizes;
    int current_size;
    Rotation current_rotation;
    int best_size;
    int status;
    
    display = gdk_display_get_default();
    xdisplay = gdk_x11_display_get_xdisplay(display);
    
    if (!XRRQueryExtension(xdisplay, &event_base, &error_base) ||
        XRRQueryVersion(xdisplay, &major, &minor) == 0) {
        g_message("Error querying xrandr version\n");
        return;
    }

    if (major != 1 || minor < 1) {
        g_message("Unsupported XRandR Version(%d.%d)", major, minor);
        return;
    }

    screen = gdk_screen_get_default();
    root_window = gdk_screen_get_root_window(screen);

    config = XRRGetScreenInfo(xdisplay, 
                              gdk_x11_drawable_get_xid(GDK_DRAWABLE(root_window)));
    sizes = XRRConfigSizes(config, &n_sizes);

    best_size = bestsize(sizes, n_sizes, width, height);
    current_size = XRRConfigCurrentConfiguration(config, &current_rotation);

    if (best_size != current_size) {
        status = XRRSetScreenConfig(xdisplay,
                                    config,
                                    gdk_x11_drawable_get_xid(GDK_DRAWABLE(root_window)),
                                    best_size,
                                    current_rotation,
                                    GDK_CURRENT_TIME);
        saved_w = sizes[current_size].width;
        saved_h = sizes[current_size].height;
        gdk_display_sync(display);
    }
#endif
}
    
void restore_rr_mode()
{
#ifdef HAVE_RANDR
    GdkDisplay *display;
    Display *xdisplay;
    int major, minor;
    int event_base, error_base;
    GdkScreen *screen;
    GdkWindow *root_window;
    XRRScreenConfiguration *config;
    XRRScreenSize *sizes;
    Rotation current_rotation;
    int n_sizes;
    int current_size;
    int best_size;
    int status;

    if (saved_w < 0) {
        return;
    }

    display = gdk_display_get_default();
    xdisplay = gdk_x11_display_get_xdisplay(display);
    
    if (!XRRQueryExtension(xdisplay, &event_base, &error_base) ||
        XRRQueryVersion(xdisplay, &major, &minor) == 0) {
        return;
    }

    if (major != 1 || minor < 1) {
        g_message("Unsupported XRandR Version(%d.%d)", major, minor);
        return;
    }

    screen = gdk_screen_get_default();
    root_window = gdk_screen_get_root_window(screen);

    config = XRRGetScreenInfo(xdisplay, 
                              gdk_x11_drawable_get_xid(GDK_DRAWABLE(root_window)));
    sizes = XRRConfigSizes(config, &n_sizes);

    best_size = bestsize(sizes, n_sizes, saved_w, saved_h);

    current_size = XRRConfigCurrentConfiguration(config, &current_rotation);
    if (best_size != current_size) {
        status = XRRSetScreenConfig(xdisplay,
                                    config,
                                    gdk_x11_drawable_get_xid(GDK_DRAWABLE(root_window)),
                                    best_size,
                                    current_rotation,
                                    GDK_CURRENT_TIME);
        gdk_display_sync(display);
    }
#endif    
}  
    

        
